from pic10f200dip8 import *
from pic10f200sip6 import *
from pic10f202dip8 import *
from pic10f202sip6 import *
from pic10f320dip8 import *
from pic10f320sip6 import *
from pic10f322dip8 import *
from pic10f322sip6 import *
from pic12f1501dip8 import *
from pic12f1501sip6 import *
from pic12f1822dip8 import *
from pic12f1822sip6 import *
from pic12f508dip8 import *
from pic12f508sip6 import *
from pic12f629dip8 import *
from pic12f629sip6 import *
from pic16f1454dip14 import *
from pic16f1454sip6 import *
from pic16f1459dip20 import *
from pic16f1459sip6 import *
from pic16f1503dip14 import *
from pic16f1503sip6 import *
from pic16f1507dip20 import *
from pic16f1507sip6 import *
from pic16f1508dip20 import *
from pic16f1508sip6 import *
from pic16f1509dip20 import *
from pic16f1509sip6 import *
from pic16f1824dip14 import *
from pic16f1824sip6 import *
from pic16f1825dip14 import *
from pic16f1825sip6 import *
from pic16f1826dip18 import *
from pic16f1826sip6 import *
from pic16f1827dip18 import *
from pic16f1827sip6 import *
from pic16f1828dip20 import *
from pic16f1828sip6 import *
from pic16f1829dip20 import *
from pic16f1829sip6 import *
from pic16f1933dip28 import *
from pic16f1933sip6 import *
from pic16f1934dip40 import *
from pic16f1934sip6 import *
from pic16f1936dip28 import *
from pic16f1936sip6 import *
from pic16f1937dip40 import *
from pic16f1937sip6 import *
from pic16f1938dip28 import *
from pic16f1938sip6 import *
from pic16f1939dip40 import *
from pic16f1939sip6 import *
from pic16f59dip40 import *
from pic16f59sip6 import *
from pic16f630dip14 import *
from pic16f630sip6 import *
from pic16f84adip18 import *
from pic16f84asip6 import *
from pic16lf1902dip28 import *
from pic16lf1902sip6 import *
from pic16lf1903dip28 import *
from pic16lf1903sip6 import *
from pic16lf1904dip40 import *
from pic16lf1904sip6 import *
from pic16lf1906dip28 import *
from pic16lf1906sip6 import *
from pic16lf1907dip40 import *
from pic16lf1907sip6 import *
from pic18f1220dip18 import *
from pic18f1220sip6 import *
from pic18f2320dip28 import *
from pic18f2320sip6 import *
from pic18f2321dip28 import *
from pic18f2321sip6 import *
#from pic18f67j60sip6 import *
